/*
   This code was made by the group DUCKTAPE for the Final assignment of smart environment in Module 2 of CreaTe
   This code uses 2 pulse sensors to measure the heart rate and use it to calculate a sleepiness value
   Afther this sleepiness value is calculated it will give a response to the driver in the form of a screen changing colour and/or a speaker making an annoying sound
   The heart rate and sleepiness value are then being send to processing via serial communication to be stored inside a csv file
   This file can then be used to graph the sleepiness of the person driving
   When the driver becomes aware they are sleepy they can respond to this.
   This code uses the PulseSensorPlayground and TFT library
*/

#define USE_ARDUINO_INTERRUPTS true
#include <PulseSensorPlayground.h>
#include <TFT.h>

#define cs   10
#define dc   9
#define rst  8

TFT TFTscreen = TFT(cs, dc, rst);

const int PULSE_SENSOR_COUNT = 2;
const int PULSE_INPUT0 = A0;
const int PULSE_INPUT1 = A1;
const int THRESHOLD = 550;
int average = 100;

PulseSensorPlayground pulseSensor(PULSE_SENSOR_COUNT);

int speaker = 0;//pin of the speaker
float Endvalue; //The sleepiness value

//Serial Communication protocol
char headers[] = {'A', 'B', 'C', 'X', 'Y', 'Z'};
void setup() {
  Serial.begin(9600);
  Serial.println("Start");

  TFTscreen.begin();
  TFTscreen.background(0, 0, 0); //Start with a black screen
  pulseSensor.analogInput(PULSE_INPUT0, 0);
  pulseSensor.analogInput(PULSE_INPUT1, 1);
  pulseSensor.setThreshold(THRESHOLD);
  if (pulseSensor.begin()) {
  }
}

void loop() {
  delay(1000);
  if (!Endvalue < 20) {
    TFTscreen.background(20, 20, 20);//Dark grey
  }
  
  if (Endvalue >= 20 && Endvalue < 40) {
    TFTscreen.background(0, 255, 0); //Green
  }
  
  if (Endvalue >= 40 && Endvalue < 60) {
    TFTscreen.background(0, 255, 255); //Yellow
    for (int j = 0; j < 2; j++) {
      tone(speaker, 1000);
      delay(50);
      noTone(speaker);
      delay(50);
    }
  }
  
  if (Endvalue >= 60 && Endvalue < 80) {
    TFTscreen.background(0, 185, 255); //Orange
    for (int j = 0; j < 3; j++) {
      tone(speaker, 1500);
      delay(50);
      noTone(speaker);
      delay(50);
    }
  }
  
  if (Endvalue >= 80 && Endvalue < 100) {
    TFTscreen.background(0, 0, 255); //Red
    for (int j = 0; j < 4; j++) {
      tone(speaker, 2000);
      delay(10);
      noTone(speaker);
      delay(10);
    }
  }
  
  if (Endvalue >= 100) {
    TFTscreen.background(0, 0, 255); //Blinking between white and red
    boolean color = false;
    for (int j = 0; j < 5; j++) {
      tone(speaker, 2000);
      delay(50);
      noTone(speaker);
      if (color) {
        TFTscreen.background(255, 255, 255);
        color = false;
      } else {
        TFTscreen.background(0, 0, 255);
        color = true;
      }
    }
  }
  int myBPM1 = pulseSensor.getBeatsPerMinute(0);
  int myBPM2 = pulseSensor.getBeatsPerMinute(1);
  average = (myBPM1 + myBPM2) / PULSE_SENSOR_COUNT;
  Endvalue = (calculate(average, 100, 60)); //Calculate the sleepiness
  if (pulseSensor.sawStartOfBeat()) {
  }

  //Data transfer to processing
  float pins[] = {float(average), Endvalue}; //heartrate, breathing rate, calc heartrate, calc breathrate, sleepiness
  Serial.print(headers[0]);
  Serial.print(average);
  Serial.print(headers[1]);
  Serial.println(Endvalue);
}

//Formula to calculate the sleepiness
float calculate(float x1, float sampleVal, float lowerBound) {
  float value = (100.0 * (pow(0.1, (x1 / (sampleVal - lowerBound))))) / (pow(0.1, lowerBound / (sampleVal - lowerBound)));
  return value;
}
